@@ -4,16 +4,6 @@ The module contains classes used to store information parsed from JPEG 2000
 codestreams.
 """
 
-# The number of lines in the module is long and that's ok.  It would not help
-# matters to move anything out to another file.
-
-# "Too many instance attributes", "Too many arguments"
-# Some segments just have a lot of information.
-# It doesn't make sense to subclass just for that.
-
-# "Too few public methods"  Some segments don't define any new methods from
-# the base Segment class.
-
 import math
 import struct
 import sys
@@ -23,21 +13,22 @@ import numpy as np
 
 from .core import (LRCP, RLCP, RPCL, PCRL, CPRL,
                    WAVELET_XFORM_9X7_IRREVERSIBLE,
-                   WAVELET_XFORM_5X3_REVERSIBLE,
-                   _Keydefaultdict)
+                   WAVELET_XFORM_5X3_REVERSIBLE)
 from .lib import openjp2 as opj2
 
 
-_factory = lambda x: '{0} (invalid)'.format(x)
-_PROGRESSION_ORDER_DISPLAY = _Keydefaultdict(_factory, {LRCP: 'LRCP',
-                                                        RLCP: 'RLCP',
-                                                        RPCL: 'RPCL',
-                                                        PCRL: 'PCRL',
-                                                        CPRL: 'CPRL'})
+_PROGRESSION_ORDER_DISPLAY = {
+    LRCP: 'LRCP',
+    RLCP: 'RLCP',
+    RPCL: 'RPCL',
+    PCRL: 'PCRL',
+    CPRL: 'CPRL',
+}
 
-_keysvalues = {WAVELET_XFORM_9X7_IRREVERSIBLE: '9-7 irreversible',
-               WAVELET_XFORM_5X3_REVERSIBLE: '5-3 reversible'}
-_WAVELET_TRANSFORM_DISPLAY = _Keydefaultdict(_factory, _keysvalues)
+_WAVELET_TRANSFORM_DISPLAY = {
+    WAVELET_XFORM_9X7_IRREVERSIBLE: '9-7 irreversible',
+    WAVELET_XFORM_5X3_REVERSIBLE: '5-3 reversible'
+}
 
 _NO_PROFILE = 0
 _PROFILE_0 = 1
@@ -48,11 +39,13 @@ _PROFILE_4 = 4
 _KNOWN_PROFILES = [_NO_PROFILE, _PROFILE_0, _PROFILE_1, _PROFILE_3, _PROFILE_4]
 
 # How to display the codestream profile.
-_CAPABILITIES_DISPLAY = _Keydefaultdict(_factory, {_NO_PROFILE: 'no profile',
-                                                   _PROFILE_0: '0',
-                                                   _PROFILE_1: '1',
-                                                   _PROFILE_3: 'Cinema 2K',
-                                                   _PROFILE_4: 'Cinema 4K'})
+_CAPABILITIES_DISPLAY = {
+    _NO_PROFILE: 'no profile',
+    _PROFILE_0: '0',
+    _PROFILE_1: '1',
+    _PROFILE_3: 'Cinema 2K',
+    _PROFILE_4: 'Cinema 4K',
+}
 
 # Need a catch-all list of valid markers.
 # See table A-1 in ISO/IEC FCD15444-1.
@@ -90,6 +83,12 @@ class Codestream(object):
        15444-1:2004 - Information technology -- JPEG 2000 image coding system:
        Core coding system
     """
+    # These two begin their lives as class attributes that usually become
+    # instance attributes.  The reason why isn't important for users; it's only
+    # important for testing purposes.
+    _csiz = -1  # Number of components in the image.
+    _parse_tpart_flag = False  # Do we parse the bit stream for SOP / EPH?
+
     def __init__(self, fptr, length, header_only=True):
         """
         Parameters
@@ -103,74 +102,67 @@ class Codestream(object):
             Supplying False may impose a large performance penalty.
         """
         # Map each of the known markers to a method that processes them.
-        process_marker_segment = {0xff00:  self._parse_reserved_segment,
-                                  0xff01:  self._parse_reserved_segment,
-                                  0xff30:  self._parse_reserved_marker,
-                                  0xff31:  self._parse_reserved_marker,
-                                  0xff32:  self._parse_reserved_marker,
-                                  0xff33:  self._parse_reserved_marker,
-                                  0xff34:  self._parse_reserved_marker,
-                                  0xff35:  self._parse_reserved_marker,
-                                  0xff36:  self._parse_reserved_marker,
-                                  0xff37:  self._parse_reserved_marker,
-                                  0xff38:  self._parse_reserved_marker,
-                                  0xff39:  self._parse_reserved_marker,
-                                  0xff3a:  self._parse_reserved_marker,
-                                  0xff3b:  self._parse_reserved_marker,
-                                  0xff3c:  self._parse_reserved_marker,
-                                  0xff3d:  self._parse_reserved_marker,
-                                  0xff3e:  self._parse_reserved_marker,
-                                  0xff3f:  self._parse_reserved_marker,
-                                  0xff4f:  self._parse_reserved_segment,
-                                  0xff50:  self._parse_reserved_segment,
-                                  0xff51:  self._parse_siz_segment,
-                                  0xff52:  self._parse_cod_segment,
-                                  0xff53:  self._parse_coc_segment,
-                                  0xff54:  self._parse_reserved_segment,
-                                  0xff55:  self._parse_tlm_segment,
-                                  0xff56:  self._parse_reserved_segment,
-                                  0xff57:  self._parse_reserved_segment,
-                                  0xff58:  self._parse_plt_segment,
-                                  0xff59:  self._parse_reserved_segment,
-                                  0xff5a:  self._parse_reserved_segment,
-                                  0xff5b:  self._parse_reserved_segment,
-                                  0xff5c:  self._parse_qcd_segment,
-                                  0xff5d:  self._parse_qcc_segment,
-                                  0xff5e:  self._parse_rgn_segment,
-                                  0xff5f:  self._parse_pod_segment,
-                                  0xff60:  self._parse_ppm_segment,
-                                  0xff61:  self._parse_ppt_segment,
-                                  0xff62:  self._parse_reserved_segment,
-                                  0xff63:  self._parse_crg_segment,
-                                  0xff64:  self._parse_cme_segment,
-                                  0xff65:  self._parse_reserved_segment,
-                                  0xff66:  self._parse_reserved_segment,
-                                  0xff67:  self._parse_reserved_segment,
-                                  0xff68:  self._parse_reserved_segment,
-                                  0xff69:  self._parse_reserved_segment,
-                                  0xff6a:  self._parse_reserved_segment,
-                                  0xff6b:  self._parse_reserved_segment,
-                                  0xff6c:  self._parse_reserved_segment,
-                                  0xff6d:  self._parse_reserved_segment,
-                                  0xff6e:  self._parse_reserved_segment,
-                                  0xff6f:  self._parse_reserved_segment,
-                                  0xff79:  self._parse_unrecognized_segment,
-                                  0xff90:  self._parse_sot_segment,
-                                  0xff91:  self._parse_unrecognized_segment,
-                                  0xff92:  self._parse_unrecognized_segment,
-                                  0xff93:  self._parse_sod_segment,
-                                  0xffd9:  self._parse_eoc_segment}
+        process_marker_segment = {0xff00: self._parse_reserved_segment,
+                                  0xff01: self._parse_reserved_segment,
+                                  0xff30: self._parse_reserved_marker,
+                                  0xff31: self._parse_reserved_marker,
+                                  0xff32: self._parse_reserved_marker,
+                                  0xff33: self._parse_reserved_marker,
+                                  0xff34: self._parse_reserved_marker,
+                                  0xff35: self._parse_reserved_marker,
+                                  0xff36: self._parse_reserved_marker,
+                                  0xff37: self._parse_reserved_marker,
+                                  0xff38: self._parse_reserved_marker,
+                                  0xff39: self._parse_reserved_marker,
+                                  0xff3a: self._parse_reserved_marker,
+                                  0xff3b: self._parse_reserved_marker,
+                                  0xff3c: self._parse_reserved_marker,
+                                  0xff3d: self._parse_reserved_marker,
+                                  0xff3e: self._parse_reserved_marker,
+                                  0xff3f: self._parse_reserved_marker,
+                                  0xff4f: self._parse_reserved_segment,
+                                  0xff50: self._parse_reserved_segment,
+                                  0xff51: self._parse_siz_segment,
+                                  0xff52: self._parse_cod_segment,
+                                  0xff53: self._parse_coc_segment,
+                                  0xff54: self._parse_reserved_segment,
+                                  0xff55: self._parse_tlm_segment,
+                                  0xff56: self._parse_reserved_segment,
+                                  0xff57: self._parse_reserved_segment,
+                                  0xff58: self._parse_plt_segment,
+                                  0xff59: self._parse_reserved_segment,
+                                  0xff5a: self._parse_reserved_segment,
+                                  0xff5b: self._parse_reserved_segment,
+                                  0xff5c: self._parse_qcd_segment,
+                                  0xff5d: self._parse_qcc_segment,
+                                  0xff5e: self._parse_rgn_segment,
+                                  0xff5f: self._parse_pod_segment,
+                                  0xff60: self._parse_ppm_segment,
+                                  0xff61: self._parse_ppt_segment,
+                                  0xff62: self._parse_reserved_segment,
+                                  0xff63: self._parse_crg_segment,
+                                  0xff64: self._parse_cme_segment,
+                                  0xff65: self._parse_reserved_segment,
+                                  0xff66: self._parse_reserved_segment,
+                                  0xff67: self._parse_reserved_segment,
+                                  0xff68: self._parse_reserved_segment,
+                                  0xff69: self._parse_reserved_segment,
+                                  0xff6a: self._parse_reserved_segment,
+                                  0xff6b: self._parse_reserved_segment,
+                                  0xff6c: self._parse_reserved_segment,
+                                  0xff6d: self._parse_reserved_segment,
+                                  0xff6e: self._parse_reserved_segment,
+                                  0xff6f: self._parse_reserved_segment,
+                                  0xff79: self._parse_unrecognized_segment,
+                                  0xff90: self._parse_sot_segment,
+                                  0xff91: self._parse_unrecognized_segment,
+                                  0xff92: self._parse_unrecognized_segment,
+                                  0xff93: self._parse_sod_segment,
+                                  0xffd9: self._parse_eoc_segment}
 
         self.offset = fptr.tell()
         self.length = length
 
-        # Number of components.  Must be kept track of for the processing of
-        # many segments.
-        self._csiz = -1
-
-        # Do we parse the tile part bit stream or not?
-        self._parse_tpart_flag = False
-
         self.segment = []
 
         # First two bytes are the SOC marker.  We already know that.
@@ -184,7 +176,15 @@ class Codestream(object):
         while True:
 
             read_buffer = fptr.read(2)
-            self._marker_id, = struct.unpack('>H', read_buffer)
+            try:
+                self._marker_id, = struct.unpack('>H', read_buffer)
+            except struct.error:
+                offset = fptr.tell() - 2
+                msg = ('Invalid codestream, expected to find a marker '
+                       'at byte position {offset}.')
+                msg = msg.format(offset=offset)
+                raise IOError(msg)
+
             self._offset = fptr.tell() - 2
 
             if self._marker_id == 0xff90 and header_only:
@@ -195,9 +195,10 @@ class Codestream(object):
             try:
                 segment = process_marker_segment[self._marker_id](fptr)
             except KeyError:
-                msg = 'Invalid marker id encountered at byte {0:d} '
-                msg += 'in codestream:  "0x{1:x}"'
-                msg = msg.format(self._offset, self._marker_id)
+                msg = ('Invalid marker ID 0x{marker_id:x} encountered at byte '
+                       '{offset:d}.')
+                msg = msg.format(offset=self._offset,
+                                 marker_id=self._marker_id)
                 warnings.warn(msg, UserWarning)
                 break
 
@@ -215,13 +216,16 @@ class Codestream(object):
                     self._parse_tile_part_bit_stream(fptr, segment,
                                                      self._tile_length[-1])
 
-                fptr.seek(self._tile_offset[-1] + self._tile_length[-1])
+                new_offset = self._tile_offset[-1] + self._tile_length[-1]
+                fptr.seek(new_offset)
 
     def _parse_unrecognized_segment(self, fptr):
         """Looks like a valid marker, but not sure from reading the specs.
         """
-        msg = "Unrecognized marker id:  0x{0:x}".format(self._marker_id)
-        warnings.warn(msg)
+        msg = ("Unrecognized codestream marker 0x{marker_id:x} encountered at "
+               "byte offset {offset}.")
+        msg = msg.format(marker_id=self._marker_id, offset=fptr.tell())
+        warnings.warn(msg, UserWarning)
         cpos = fptr.tell()
         read_buffer = fptr.read(2)
         next_item, = struct.unpack('>H', read_buffer)
@@ -253,7 +257,7 @@ class Codestream(object):
         read_buffer = fptr.read(2)
         length, = struct.unpack('>H', read_buffer)
         if length > 0:
-            data = fptr.read(length-2)
+            data = fptr.read(length - 2)
         else:
             data = None
 
@@ -272,10 +276,10 @@ class Codestream(object):
         indices = np.where(packet == 0xff)
         for idx in indices[0]:
             try:
-                if packet[idx+1] == 0x91 and (idx < (len(packet) - 5)):
+                if packet[idx + 1] == 0x91 and (idx < (len(packet) - 5)):
                     offset = sod_marker.offset + 2 + idx
                     length = 4
-                    nsop = packet[(idx + 4):(idx+6)].view('uint16')[0]
+                    nsop = packet[(idx + 4):(idx + 6)].view('uint16')[0]
                     if sys.byteorder == 'little':
                         nsop = nsop.byteswap()
                     segment = SOPsegment(nsop, length, offset)
@@ -296,7 +300,7 @@ class Codestream(object):
             # Add indentation
             strs = [('    ' + x + '\n') for x in strs.split('\n')]
             msg += ''.join(strs)
-        return msg
+        return msg.rstrip()
 
     def _parse_cme_segment(self, fptr):
         """Parse the CME marker segment.
@@ -360,7 +364,8 @@ class Codestream(object):
 
         return COCsegment(ccoc, scoc, spcoc, length, offset)
 
-    def _parse_cod_segment(self, fptr):
+    @classmethod
+    def _parse_cod_segment(cls, fptr):
         """Parse the COD segment.
 
         Parameters
@@ -379,27 +384,27 @@ class Codestream(object):
         length, = struct.unpack('>H', read_buffer)
 
         read_buffer = fptr.read(length - 2)
-        scod, = struct.unpack_from('>B', read_buffer, offset=0)
-        spcod = read_buffer[1:]
-        spcod = np.frombuffer(spcod, dtype=np.uint8)
-        if spcod[0] not in [LRCP, RLCP, RPCL, PCRL, CPRL]:
-            msg = "Invalid progression order in COD segment: {0}."
-            warnings.warn(msg.format(spcod[0]))
 
-        if spcod[8] not in [WAVELET_XFORM_9X7_IRREVERSIBLE,
-                            WAVELET_XFORM_5X3_REVERSIBLE]:
-            msg = "Invalid wavelet transform in COD segment: {0}."
-            warnings.warn(msg.format(spcod[8]))
+        lst = struct.unpack_from('>BBHBBBBBB', read_buffer, offset=0)
+        scod, prog, nlayers, mct, nr, xcb, ycb, cstyle, xform = lst
+
+        if len(read_buffer) > 10:
+            precinct_size = _parse_precinct_size(read_buffer[10:])
+        else:
+            precinct_size = None
 
         sop = (scod & 2) > 0
         eph = (scod & 4) > 0
 
         if sop or eph:
-            self._parse_tpart_flag = True
+            cls._parse_tpart_flag = True
         else:
-            self._parse_tpart_flag = False
+            cls._parse_tpart_flag = False
 
-        return CODsegment(scod, spcod, length, offset)
+        pargs = (scod, prog, nlayers, mct, nr, xcb, ycb, cstyle, xform,
+                 precinct_size)
+
+        return CODsegment(*pargs, length=length, offset=offset)
 
     def _parse_crg_segment(self, fptr):
         """Parse the CRG marker segment.
@@ -566,7 +571,8 @@ class Codestream(object):
 
         return PPTsegment(zppt, ippt, length, offset)
 
-    def _parse_qcc_segment(self, fptr):
+    @classmethod
+    def _parse_qcc_segment(cls, fptr):
         """Parse the QCC segment.
 
         Parameters
@@ -585,18 +591,18 @@ class Codestream(object):
         length, = struct.unpack('>H', read_buffer)
 
         read_buffer = fptr.read(length - 2)
-        if self._csiz > 256:
+        if cls._csiz > 256:
             fmt = '>HB'
             mantissa_exponent_offset = 3
         else:
             fmt = '>BB'
             mantissa_exponent_offset = 2
         cqcc, sqcc = struct.unpack_from(fmt, read_buffer)
-        if cqcc >= self._csiz:
-            msg = "Invalid component number ({0}), "
-            msg += "number of components is only {1}."
-            msg = msg.format(cqcc, self._csiz)
-            warnings.warn(msg)
+        if cqcc >= cls._csiz:
+            msg = ("Invalid QCC component number ({invalid_comp_no}), "
+                   "the actual number of components is only {valid_comp_no}.")
+            msg = msg.format(invalid_comp_no=cqcc, valid_comp_no=cls._csiz)
+            warnings.warn(msg, UserWarning)
 
         spqcc = read_buffer[mantissa_exponent_offset:]
 
@@ -623,7 +629,8 @@ class Codestream(object):
 
         return QCDsegment(sqcd, spqcd, length, offset)
 
-    def _parse_rgn_segment(self, fptr):
+    @classmethod
+    def _parse_rgn_segment(cls, fptr):
         """Parse the RGN segment.
 
         Parameters
@@ -641,7 +648,7 @@ class Codestream(object):
         read_buffer = fptr.read(2)
         length, = struct.unpack('>H', read_buffer)
 
-        if self._csiz < 257:
+        if cls._csiz < 257:
             read_buffer = fptr.read(3)
             data = struct.unpack('>BBB', read_buffer)
         else:
@@ -655,7 +662,8 @@ class Codestream(object):
 
         return RGNsegment(crgn, srgn, sprgn, length, offset)
 
-    def _parse_siz_segment(self, fptr):
+    @classmethod
+    def _parse_siz_segment(cls, fptr):
         """Parse the SIZ segment.
 
         Parameters
@@ -678,7 +686,8 @@ class Codestream(object):
 
         rsiz = data[0]
         if rsiz not in _KNOWN_PROFILES:
-            warnings.warn("Invalid profile: (Rsiz={0}).".format(rsiz))
+            msg = "Invalid profile: (Rsiz={rsiz}).".format(rsiz=rsiz)
+            warnings.warn(msg, UserWarning)
 
         xysiz = (data[1], data[2])
         xyosiz = (data[3], data[4])
@@ -698,38 +707,48 @@ class Codestream(object):
 
         for j, subsampling in enumerate(zip(xrsiz, yrsiz)):
             if 0 in subsampling:
-                msg = "Invalid subsampling value for component {0}: "
-                msg += "dx={1}, dy={2}."
-                msg = msg.format(j, subsampling[0], subsampling[1])
-                warnings.warn(msg)
+                msg = ("Invalid subsampling value for component {comp}: "
+                       "dx={dx}, dy={dy}.")
+                msg = msg.format(comp=j, dx=subsampling[0], dy=subsampling[1])
+                warnings.warn(msg, UserWarning)
 
         try:
             num_tiles_x = (xysiz[0] - xyosiz[0]) / (xytsiz[0] - xytosiz[0])
             num_tiles_y = (xysiz[1] - xyosiz[1]) / (xytsiz[1] - xytosiz[1])
         except ZeroDivisionError:
-            warnings.warn("Invalid tile dimensions.")
+            msg = ("Invalid tile specification:  "
+                   "size of {num_tile_rows} x {num_tile_cols}, "
+                   "offset of {row_offset} x {col_offset}.")
+            msg = msg.format(num_tile_rows=xytsiz[1],
+                             num_tile_cols=xytsiz[0],
+                             row_offset=xytosiz[1],
+                             col_offset=xytosiz[0])
+            warnings.warn(msg, UserWarning)
         else:
             numtiles = math.ceil(num_tiles_x) * math.ceil(num_tiles_y)
             if numtiles > 65535:
-                msg = "Invalid number of tiles ({0}).".format(numtiles)
-                warnings.warn(msg)
+                msg = "Invalid number of tiles: ({numtiles})."
+                msg = msg.format(numtiles=numtiles)
+                warnings.warn(msg, UserWarning)
 
-        kwargs = {'rsiz': rsiz,
-                  'xysiz': xysiz,
-                  'xyosiz': xyosiz,
-                  'xytsiz': xytsiz,
-                  'xytosiz': xytosiz,
-                  'Csiz': Csiz,
-                  'bitdepth': bitdepth,
-                  'signed':  signed,
-                  'xyrsiz': (xrsiz, yrsiz),
-                  'length': length,
-                  'offset': offset}
+        kwargs = {
+            'rsiz': rsiz,
+            'xysiz': xysiz,
+            'xyosiz': xyosiz,
+            'xytsiz': xytsiz,
+            'xytosiz': xytosiz,
+            'Csiz': Csiz,
+            'bitdepth': bitdepth,
+            'signed': signed,
+            'xyrsiz': (xrsiz, yrsiz),
+            'length': length,
+            'offset': offset
+        }
         segment = SIZsegment(**kwargs)
 
         # Need to keep track of the number of components from SIZ for
-        # other markers.
-        self._csiz = Csiz
+        # other segments.
+        cls._csiz = Csiz
 
         return segment
 
@@ -841,7 +860,7 @@ class Codestream(object):
             ttlm = data[0::2]
             ptlm = data[1::2]
 
-        return TLMsegment(length, offset, ztlm, ttlm, ptlm)
+        return TLMsegment(ztlm, ttlm, ptlm, length, offset)
 
     def _parse_reserved_marker(self, fptr):
         """Marker range between 0xff30 and 0xff39.
@@ -874,10 +893,10 @@ class Segment(object):
         self.data = data
 
     def __str__(self):
-        msg = '{0} marker segment @ ({1}, {2})'.format(self.marker_id,
-                                                       self.offset,
-                                                       self.length)
-        return msg
+        msg = '{marker_id} marker segment @ ({offset}, {length})'
+        return msg.format(marker_id=self.marker_id,
+                          length=self.length,
+                          offset=self.offset)
 
 
 class COCsegment(Segment):
@@ -919,7 +938,7 @@ class COCsegment(Segment):
         if len(self.spcoc) > 5:
             self.precinct_size = _parse_precinct_size(self.spcoc[5:])
         else:
-            self.precinct_size = None
+            self.precinct_size = ((32768, 32768))
 
         self.length = length
         self.offset = offset
@@ -927,30 +946,24 @@ class COCsegment(Segment):
     def __str__(self):
         msg = Segment.__str__(self)
 
-        msg += '\n    Associated component:  {0}'.format(self.ccoc)
-
-        msg += '\n    Coding style for this component:  '
-        if self.scoc == 0:
-            msg += 'Entropy coder, PARTITION = 0'
-        elif self.scoc & 0x01:
-            msg += 'Entropy coder, PARTITION = 1'
-
-        msg += '\n    Coding style parameters:'
-        msg += '\n        Number of resolutions:  {0}'
-        msg += '\n        Code block height, width:  ({1} x {2})'
-        msg += '\n        Wavelet transform:  {3}'
-        msg = msg.format(self.spcoc[0] + 1,
-                         int(self.code_block_size[0]),
-                         int(self.code_block_size[1]),
-                         _WAVELET_TRANSFORM_DISPLAY[self.spcoc[4]])
-
-        msg += '\n        '
-        msg += _context_string(self.spcoc[3])
-
-        if self.precinct_size is not None:
-            msg += '\n        Precinct size:  '
-            for pps in self.precinct_size:
-                msg += '(%d, %d)'.format(pps)
+        msg += '\n'
+        msg += ('    Associated component:  {assoc_comp}\n'
+                '    Coding style for this component:  '
+                'Entropy coder, PARTITION = {partition}\n'
+                '    Coding style parameters:\n'
+                '        Number of resolutions:  {num_res}\n'
+                '        Code block height, width:  ({cblh} x {cblw})\n'
+                '        Wavelet transform:  {xform}\n'
+                '        Precinct size:  {psize}\n'
+                '        {context_string}')
+        msg = msg.format(assoc_comp=self.ccoc,
+                         partition=0 if self.scoc == 0 else 1,
+                         num_res=self.spcoc[0] + 1,
+                         cblh=int(self.code_block_size[0]),
+                         cblw=int(self.code_block_size[1]),
+                         xform=_WAVELET_TRANSFORM_DISPLAY[self.spcoc[4]],
+                         context_string=_context_string(self.spcoc[3]),
+                         psize=self.precinct_size)
 
         return msg
 
@@ -961,104 +974,127 @@ class CODsegment(Segment):
     Attributes
     ----------
     marker_id : str
-        Identifier for the segment.
+        Identifier for the segment
     offset : int
-        Offset of marker segment in bytes from beginning of file.
+        Offset of marker segment in bytes from beginning of file
     length : int
         Length of marker segment in bytes.  This number does not include the
         two bytes constituting the marker.
     scod : int
-        Default coding style.
-    layers : int
-        Quality layers.
+        Default coding style
     code_block_size : tuple
-        Size of code block.
-    spcod : bytes
-        Encoded coding style parameters, including quality layers,
-        multi component transform usage, decomposition levels, code block size,
-        style of code-block passes, and which wavelet transform is used.
-    precinct_size : list of tuples
-        Dimensions of precinct.
+        Size of code block
+    layers : int
+        Number of decomposition levels
+    progression_order : int
+        Progression order
+    mct : int
+        Multiple component transform usage
+    num_res : int
+        Number of layers
+    xform : int
+        Wavelet transform used
+    cstyle : int
+        Style of the code-block passes
+    precinct_size : list
+        2-tuples of precinct sizes.
 
     References
     ----------
     .. [JP2K15444-1i] International Organization for Standardication.  ISO/IEC
        15444-1:2004 - Information technology -- JPEG 2000 image coding system:
        Core coding system
+
+        pargs = (scod, prog, nlayers, mct, nr, xcb, ycb, cstyle, xform,
+                 precinct_size)
     """
-    def __init__(self, scod, spcod, length, offset):
+    def __init__(self, scod, prog_order, num_layers, mct, nr, xcb, ycb,
+                 cstyle, xform, precinct_size, length=0, offset=0):
         Segment.__init__(self, marker_id='COD')
         self.scod = scod
-        self.spcod = spcod
         self.length = length
         self.offset = offset
+        self.mct = mct
+        self.cstyle = cstyle
+        self.xform = xform
 
-        params = struct.unpack('>BHBBBBBB', self.spcod[0:9])
-        self.layers = params[1]
-        self._numresolutions = params[3]
+        self.layers = num_layers
+        self._numresolutions = nr
 
-        if params[3] > opj2.J2K_MAXRLVLS:
-            msg = "Invalid number of resolutions ({0})."
-            msg = msg.format(params[3] + 1)
-            warnings.warn(msg)
+        if nr > opj2.J2K_MAXRLVLS:
+            msg = "Invalid number of resolutions: ({numres})."
+            msg = msg.format(numres=nr + 1)
+            warnings.warn(msg, UserWarning)
+        self.num_res = nr
 
-        cblk_width = 4 * math.pow(2, params[4])
-        cblk_height = 4 * math.pow(2, params[5])
+        if prog_order not in [LRCP, RLCP, RPCL, PCRL, CPRL]:
+            msg = "Invalid progression order in COD segment: {prog_order}."
+            warnings.warn(msg.format(prog_order=prog_order), UserWarning)
+        self.prog_order = prog_order
+
+        if xform not in [WAVELET_XFORM_9X7_IRREVERSIBLE,
+                         WAVELET_XFORM_5X3_REVERSIBLE]:
+            msg = "Invalid wavelet transform in COD segment: {xform}."
+            msg = msg.format(xform=xform)
+            warnings.warn(msg, UserWarning)
+
+        cblk_width = 4 * math.pow(2, xcb)
+        cblk_height = 4 * math.pow(2, ycb)
         code_block_size = (cblk_height, cblk_width)
         self.code_block_size = code_block_size
 
-        if len(self.spcod) > 9:
-            self.precinct_size = _parse_precinct_size(self.spcod[9:])
+        if precinct_size is None:
+            self.precinct_size = ((2 ** 15, 2 ** 15))
         else:
-            self.precinct_size = None
+            self.precinct_size = precinct_size
 
     def __str__(self):
         msg = Segment.__str__(self)
 
-        msg += '\n    Coding style:'
-        msg += '\n        Entropy coder, {0} partitions'
-        msg += '\n        SOP marker segments:  {1}'
-        msg += '\n        EPH marker segments:  {2}'
-        msg = msg.format('with' if (self.scod & 1) else 'without',
-                         ((self.scod & 2) > 0),
-                         ((self.scod & 4) > 0))
+        msg += '\n'
+        msg += ('    Coding style:\n'
+                '        Entropy coder, {with_without} partitions\n'
+                '        SOP marker segments:  {sop}\n'
+                '        EPH marker segments:  {eph}\n'
+                '    Coding style parameters:\n'
+                '        Progression order:  {prog}\n'
+                '        Number of layers:  {num_layers}\n'
+                '        Multiple component transformation usage:  {mct}\n'
+                '        Number of resolutions:  {num_resolutions}\n'
+                '        Code block height, width:  ({cbh} x {cbw})\n'
+                '        Wavelet transform:  {xform}\n'
+                '        Precinct size:  {precinct_size}\n'
+                '        {code_block_context}')
 
-        if self.spcod[3] == 0:
-            mct = 'no transform specified'
-        elif self.spcod[3] & 0x01:
-            mct = 'reversible'
-        elif self.spcod[3] & 0x02:
-            mct = 'irreversible'
+        if self.mct == 0:
+            mct_str = 'no transform specified'
+        elif self.mct & 0x01:
+            mct_str = 'reversible'
+        elif self.mct & 0x02:
+            mct_str = 'irreversible'
         else:
-            mct = 'unknown'
+            mct_str = 'unknown'
 
-        msg += '\n    '
-        lines = ['Coding style parameters:',
-                 '    Progression order:  {0}',
-                 '    Number of layers:  {1}',
-                 '    Multiple component transformation usage:  {2}',
-                 '    Number of resolutions:  {3}',
-                 '    Code block height, width:  ({4} x {5})',
-                 '    Wavelet transform:  {6}']
-        msg += '\n    '.join(lines)
-
-        msg = msg.format(_PROGRESSION_ORDER_DISPLAY[self.spcod[0]],
-                         self.layers,
-                         mct,
-                         self.spcod[4] + 1,
-                         int(self.code_block_size[0]),
-                         int(self.code_block_size[1]),
-                         _WAVELET_TRANSFORM_DISPLAY[self.spcod[8]])
-
-        msg += '\n        Precinct size:  '
-        if self.precinct_size is None:
-            msg += 'default, 2^15 x 2^15'
-        else:
-            for pps in self.precinct_size:
-                msg += '({0}, {1})'.format(pps[0], pps[1])
-
-        msg += '\n        '
-        msg += _context_string(self.spcod[7])
+        try:
+            progression_order = _PROGRESSION_ORDER_DISPLAY[self.prog_order]
+        except KeyError:
+            progression_order = '{prog} (invalid)'.format(prog=self.prog_order)
+        try:
+            xform = _WAVELET_TRANSFORM_DISPLAY[self.xform]
+        except KeyError:
+            xform = '{xform} (invalid)'.format(xform=self.xform)
+        msg = msg.format(with_without='with' if (self.scod & 1) else 'without',
+                         sop=((self.scod & 2) > 0),
+                         eph=((self.scod & 4) > 0),
+                         prog=progression_order,
+                         num_layers=self.layers,
+                         mct=mct_str,
+                         num_resolutions=self.num_res + 1,
+                         cbh=int(self.code_block_size[0]),
+                         cbw=int(self.code_block_size[1]),
+                         xform=xform,
+                         precinct_size=self.precinct_size,
+                         code_block_context=_context_string(self.cstyle))
 
         return msg
 
@@ -1098,11 +1134,10 @@ class CMEsegment(Segment):
         msg = Segment.__str__(self) + '\n'
         if self.rcme == 1:
             # latin-1 string
-            msg += '    "{0}"'
-            msg = msg.format(self.ccme.decode('latin-1'))
+            msg += '    "{ccme}"'.format(ccme=self.ccme.decode('latin-1'))
         else:
-            msg += "    binary data (rcme = {0}):  {1} bytes"
-            msg = msg.format(self.rcme, len(self.ccme))
+            msg += "    binary data (rcme = {rcme}):  {nbytes} bytes"
+            msg = msg.format(rcme=self.rcme, nbytes=len(self.ccme))
         return msg
 
 
@@ -1132,8 +1167,8 @@ class CRGsegment(Segment):
         msg = Segment.__str__(self)
         msg += '\n    Vertical, Horizontal offset: '
         for j in range(len(self.xcrg)):
-            msg += ' ({0:.2f}, {1:.2f})'.format(self.ycrg[j]/65535.0,
-                                                self.xcrg[j]/65535.0)
+            msg += ' ({0:.2f}, {1:.2f})'.format(self.ycrg[j] / 65535.0,
+                                                self.xcrg[j] / 65535.0)
         return msg
 
 
@@ -1575,25 +1610,30 @@ class SIZsegment(Segment):
 
     def __str__(self):
         msg = Segment.__str__(self)
-        msg += '\n    '
+        msg += '\n'
 
-        lines = ['Profile:  {0}',
-                 'Reference Grid Height, Width:  ({1} x {2})',
-                 'Vertical, Horizontal Reference Grid Offset:  ({3} x {4})',
-                 'Reference Tile Height, Width:  ({5} x {6})',
-                 'Vertical, Horizontal Reference Tile Offset:  ({7} x {8})',
-                 'Bitdepth:  {9}',
-                 'Signed:  {10}',
-                 'Vertical, Horizontal Subsampling:  {11}']
-        msg += '\n    '.join(lines)
-        msg = msg.format(_CAPABILITIES_DISPLAY[self.rsiz],
-                         self.ysiz, self.xsiz,
-                         self.yosiz, self.xosiz,
-                         self.ytsiz, self.xtsiz,
-                         self.ytosiz, self.xtosiz,
-                         self.bitdepth,
-                         self.signed,
-                         tuple(zip(self.yrsiz, self.xrsiz)))
+        msg += ('    Profile:  {profile}\n'
+                '    Reference Grid Height, Width:  ({height} x {width})\n'
+                '    Vertical, Horizontal Reference Grid Offset:  '
+                '({goy} x {gox})\n'
+                '    Reference Tile Height, Width:  ({tileh} x {tilew})\n'
+                '    Vertical, Horizontal Reference Tile Offset:  '
+                '({toy} x {tox})\n'
+                '    Bitdepth:  {bitdepth}\n'
+                '    Signed:  {signed}\n'
+                '    Vertical, Horizontal Subsampling:  {subsampling}')
+        try:
+            profile = _CAPABILITIES_DISPLAY[self.rsiz]
+        except KeyError:
+            profile = '{rsiz} (invalid)'.format(rsiz=self.rsiz)
+        msg = msg.format(profile=profile,
+                         height=self.ysiz, width=self.xsiz,
+                         goy=self.yosiz, gox=self.xosiz,
+                         tileh=self.ytsiz, tilew=self.xtsiz,
+                         toy=self.ytosiz, tox=self.xtosiz,
+                         bitdepth=self.bitdepth,
+                         signed=self.signed,
+                         subsampling=tuple(zip(self.yrsiz, self.xrsiz)))
 
         return msg
 
@@ -1790,7 +1830,7 @@ class TLMsegment(Segment):
        15444-1:2004 - Information technology -- JPEG 2000 image coding system:
        Core coding system
     """
-    def __init__(self, length, offset, ztlm, ttlm, ptlm):
+    def __init__(self, ztlm, ttlm, ptlm, length, offset):
         Segment.__init__(self, marker_id='TLM')
         self.length = length
         self.offset = offset
@@ -1801,13 +1841,13 @@ class TLMsegment(Segment):
     def __str__(self):
         msg = Segment.__str__(self)
         msg += '\n    '
-        lines = ['Index:  {0}',
-                 'Tile number:  {1}',
-                 'Length:  {2}']
+        lines = ['Index:  {index}',
+                 'Tile number:  {tile_number}',
+                 'Length:  {tile_length}']
         msg += '\n    '.join(lines)
-        msg = msg.format(self.ztlm,
-                         self.ttlm,
-                         self.ptlm)
+        msg = msg.format(index=self.ztlm,
+                         tile_number=self.ttlm,
+                         tile_length=self.ptlm)
 
         return msg
 
@@ -1820,7 +1860,7 @@ def _parse_precinct_size(spcod):
         ep2 = (item & 0xF0) >> 4
         ep1 = item & 0x0F
         precinct_size.append((2 ** ep1, 2 ** ep2))
-    return precinct_size
+    return tuple(precinct_size)
 
 
 def _context_string(context):
